
// ===========================================================================

//  #########################################################################
//  # This file contains C++ code but it is named using the C standard ".c" #
//  # file extension and not the C++ ".cpp" extension. This is done to so   #
//  # that the syntax highlighting and intellisense works correctly in the  #
//  # PSoc Creator environment. To ensure that this file compiles the C++   #
//  # code correctly in the PSoC Creator environment, the Build Settings    #
//  # Command Line setting for this file must contain the additional        #
//  # "-x c++" flag.                                                        #
//  #########################################################################

// ===========================================================================
// INCLUDES
// ---------------------------------------------------------------------------
#include "TbusSpiSlavePort.h"

// Provide for access to standard C files from C++
#ifdef __cplusplus
extern "C"{
#endif

#include "project.h"

/* RTOS includes. */
#include "FreeRTOS.h"
#include "task.h"
#include "queue.h"
#include "semphr.h"

#ifdef __cplusplus
}
#endif

// ===========================================================================
// STATIC INITIALISATIONS
// ---------------------------------------------------------------------------
TbusSpiSlavePort* TbusSpiSlavePort::InstancePtr;

// ===========================================================================
// EXTERNALS
// ---------------------------------------------------------------------------

// ---------------------------------------------------------------------------
extern "C" void TbusSpiSlavePortIsrWrapper(void)
{
  
  if (TbusSpiSlavePort::InstancePtr != 0)
    (*(TbusSpiSlavePort::InstancePtr)).TbusSpiSlavePortIsr();

}

// ---------------------------------------------------------------------------
extern "C" void TbusIoPortIsrWrapper(void)
{
  
  if (TbusSpiSlavePort::InstancePtr != 0)
    (*(TbusSpiSlavePort::InstancePtr)).TbusIoPortIsr();

}

// ===========================================================================
// CONSTANTS
// ---------------------------------------------------------------------------

// ===========================================================================
// FUNCTIONS
// ---------------------------------------------------------------------------
  
// ---------------------------------------------------------------------------
TbusSpiSlavePort::TbusSpiSlavePort()
{
}

// ------------------------------------------------------------------------
void TbusSpiSlavePort::Activate (SpiRegisterMap* spiRegisterMapPtr)
{
  TbusSpiSlavePort::InstancePtr = this;
  this->spiRegisterMapPtr = spiRegisterMapPtr;
  rxFrameParseIndex = 0;
  
  // Connect the SPI ISR, enable the interrupt, and start the SPI hardware
  Cy_SCB_SPI_Init(TbusSpi_HW, &TbusSpi_config, NULL); // Initialize SCB for SPI operation with GUI selected settings
  
  Cy_SysInt_Init(&TbusSpiIrq_cfg, &TbusSpiSlavePortIsrWrapper); // Set the IRQ ISR vector
  NVIC_EnableIRQ(TbusSpiIrq_cfg.intrSrc);
  
  // Connect the SPI Select pin IRQ (Note that this is only used for parsing in-comming frames)
	Cy_SysInt_Init(&TbusSelectIrq_cfg, TbusIoPortIsrWrapper);
	NVIC_ClearPendingIRQ(TbusSelectIrq_cfg.intrSrc);
	NVIC_EnableIRQ(TbusSelectIrq_cfg.intrSrc);
  
  Cy_SCB_SPI_Enable(TbusSpi_HW);
  
  // Initilise the PWM that pulses the SPI Comms LED
  (void) Cy_TCPWM_PWM_Init(SpiCommsLedPwm_HW, SpiCommsLedPwm_CNT_NUM, &SpiCommsLedPwm_config);
  Cy_TCPWM_Enable_Multiple(SpiCommsLedPwm_HW, SpiCommsLedPwm_CNT_MASK);  
  
 
// DEBUG ONLY  
testValue = 1;  
Cy_GPIO_Write(TestPin0_4_0_PORT, TestPin0_4_0_NUM, 1);  
Cy_GPIO_Write(TestPin0_5_0_PORT, TestPin0_5_0_NUM, 1);
Cy_GPIO_Write(TestPin5_4_0_PORT, TestPin5_4_0_NUM, 1);
  
  
}

// ---------------------------------------------------------------------------
void TbusSpiSlavePort::TbusSpiSlavePortIsr()
{  
  // This ISR implements a framing scheme. The first byte of a frame has a 
  // 7-bit register address in the lowest 7 bits, the highest bit is a 0 for 
  // register writes and a 1 for register reads. Bytes are read into rxBuffer 
  // until at least two bytes are received. 
  
  
Cy_GPIO_Write(TestPin0_5_0_PORT, TestPin0_5_0_NUM, 0);


  volatile uint32_t readCount = Cy_SCB_SPI_ReadArray(TbusSpi_HW, rxBuffer, rxBufferSize);  
  Cy_SCB_SPI_ClearRxFifoStatus(TbusSpi_HW, CY_SCB_SPI_RX_INTR_MASK);
  Cy_SCB_SPI_ClearRxFifo(TbusSpi_HW);
 
  
  // We are only interested in processing this further if the SPI select line is still asserted, and at least one byte was read.
  if ((Cy_GPIO_Read(TbusS0n_0_PORT, TbusS0n_0_NUM) == 0UL) && (readCount > 0)) 
  {
    
    // Parse all the received bytes
    for (uint32_t i = 0; i < readCount; i++)
    {
      
      // Is this the first byte of a new frame?
      if (rxFrameParseIndex == 0)
      {
        // Yes. Set up for a new read or a write
        isAReadFrame = (rxBuffer[0] & 0x80) > 0; // Is a read frame if the MSB is a 1, otherwise it's a write frame
        targetRegisterAddress = rxBuffer[0] & 0x7F;
      }
      
      // For read frames, respond immediately for each byte received because the SPI clock is running and the next clock cycle will expect to clock data out on MISO
      if (isAReadFrame)
      {

        
//        volatile uint8_t value = spiRegisterMapPtr->ReadRegister(targetRegisterAddress);   // This is non-blocking
//        Cy_SCB_SPI_Write(TbusSpi_HW, value); 
 
        
// DEBUG        
Cy_SCB_SPI_Write(TbusSpi_HW, testValue++); 


Cy_GPIO_Write(TestPin5_4_0_PORT, TestPin5_4_0_NUM, 0);
        
        // Prepare for the possiblity of there being another read in a chained (i.e. multiple byte) read frame
        targetRegisterAddress++;
      }
      else
      {
        // This is a write frame. Write receved data bytes if we have any
        
        if (rxFrameParseIndex > 0)
        {
          spiRegisterMapPtr->WriteRegisterFromIsr(targetRegisterAddress, rxBuffer[i]); // This is non-blocking
          
          // Prepare for the possiblity of there being another write in a chained (i.e. multiple byte) write frame
          targetRegisterAddress++;
        }

      }
            
      rxFrameParseIndex++;      
    }        
    
    // Pulse the SPI Comms LED (the period of the pulse is set in the GUI component)
    Cy_TCPWM_TriggerReloadOrIndex(SpiCommsLedPwm_HW, SpiCommsLedPwm_CNT_MASK);  // Re-start the pulse incase it's already running
    Cy_TCPWM_TriggerStart(SpiCommsLedPwm_HW, SpiCommsLedPwm_CNT_MASK);          // Start the pulse. This does nothing if it's already runnin
  }
       
  
Cy_GPIO_Write(TestPin5_4_0_PORT, TestPin5_4_0_NUM, 1);
Cy_GPIO_Write(TestPin0_5_0_PORT, TestPin0_5_0_NUM, 1);
  
}

// ---------------------------------------------------------------------------
void TbusSpiSlavePort::TbusIoPortIsr()
{  
  
  // Is this the interrupt from the current configured select line?
  if (Cy_GPIO_GetInterruptStatus(TbusS0n_0_PORT, TbusS0n_0_NUM) == 1UL)
  {
    // Yes.
    
    
    // Was this IRQ caused by a rising or falling edge?
    if (Cy_GPIO_Read(TbusS0n_0_PORT, TbusS0n_0_NUM) == 0UL)
    {
      // Falling Edge. A frame reception has started
      Cy_SCB_SPI_ClearTxFifoStatus(TbusSpi_HW, CY_SCB_SPI_TX_INTR_MASK );     
      Cy_SCB_SPI_ClearTxFifo(TbusSpi_HW);
      
Cy_GPIO_Write(TestPin0_4_0_PORT, TestPin0_4_0_NUM, 0);

    }
    else
    {
      // Rising Edge. A frame reception has completed
      

Cy_GPIO_Write(TestPin0_4_0_PORT, TestPin0_4_0_NUM, 1);


    }
    
testValue = 1;  
    
    
    rxFrameParseIndex = 0;

    Cy_GPIO_ClearInterrupt(TbusS0n_0_PORT, TbusS0n_0_NUM);
  }
  
}

// ===========================================================================
